@@ -6,6 +6,7 @@ import click
 from .util import get_systype
 from .examples import Examples
 from .execute import SCons, System
+from .project import Project
 from .packages.scons import SconsInstaller
 from .packages.icestorm import IcestormInstaller
 from .packages.driver import DriverInstaller
@@ -37,35 +38,39 @@ def debug():
     print('Platform: ' + get_systype())
 
 
-@cli.command('init')
-def init():
+@cli.command('scons')
+def scons():
     """Create default SConstruct file."""
     SCons().create_sconstruct()
 
 
+@cli.command('init')
+@click.pass_context
+@click.option('--board', type=unicode, help='Set the FPGA board')
+def init(ctx, board):
+    """Create a new apio project."""
+    Project().new(board)
+
+
 @cli.command('examples')
 @click.pass_context
-@click.option('-l', '--list', is_flag=True, help='List all available examples.')
-@click.option('-d', '--dir', type=unicode, help='Copy the selected example directory.',
-              metavar='NAME')
-@click.option('-f', '--files', type=unicode, help='Copy the selected example files.',
-              metavar='NAME')
+@click.option('-l', '--list', is_flag=True,
+              help='List all available examples.')
+@click.option('-d', '--dir', type=unicode, metavar='NAME',
+              help='Copy the selected example directory.')
+@click.option('-f', '--files', type=unicode, metavar='NAME',
+              help='Copy the selected example files.')
 def examples(ctx, list, dir, files):
     """Manage default verilog examples."""
     if list:
-        examples = Examples().list_examples()
-        print('')
-        for example in examples:
-            print(' > ' + example)
-        print('')
-        print('To get and example, use the command:')
-        print('apio examples -d/-f name')
+        Examples().list_examples()
     elif dir:
         Examples().copy_example_dir(dir)
     elif files:
         Examples().copy_example_files(files)
     else:
         print(ctx.get_help())
+        print(Examples().examples_of_use_cad())
 
 
 # System #
@@ -182,6 +187,20 @@ def _uninstall(*functions):
 
 
 # Synthesize #
+def format_vars(board, pack, type, size):
+    """Format the given vars in the form: 'flag=value'"""
+
+    if board is not None:
+        board = "board={}".format(board)
+    if pack is not None:
+        pack = "fpga_pack={}".format(pack)
+    if type is not None:
+        type = "fpga_type={}".format(type)
+    if size is not None:
+        size = "fpga_size={}".format(size)
+
+    vars = [f for f in [board, pack, type, size] if f is not None]
+    return vars
 
 
 @cli.command('clean')
@@ -191,15 +210,68 @@ def clean():
 
 
 @cli.command('build')
-def build():
+@click.pass_context
+@click.option('--board', type=unicode, metavar='package',
+              help='Set the FPGA board')
+@click.option('--pack', type=unicode, metavar='package',
+              help='Set the FPGA package')
+@click.option('--type', type=unicode, metavar='type',
+              help='Set the FPGA type (hx/lp)')
+@click.option('--size', type=unicode, metavar='size',
+              help='Set the FPGA type (1k/8k)')
+def build(ctx, board, pack, type, size):
     """Synthesize the bitstream."""
-    SCons().run()
+
+    # -- Get the variables and change them in the form 'flag=value'
+    vars = format_vars(board, pack, type, size)
+
+    # -- Run scons
+    SCons().run(vars)
+
+# -- Notes on the Upload target:
+# -- (Notes for advanced user)
+# --
+# -- The upload target first build the project if there was some modification
+# --   and then upload the .bin file. For that reason, upload is also a kind
+# --   of "build". The same fpga flags than build should be passed.
+# --
+# --  For example:
+# --
+# --  apio build --pack vq100
+# --
+# --  It will create the .bin file for an ice40 hx1k - vq100 FPGA
+# --  (like in the nandland go-board)
+# --
+# --  If now we upload it using:
+# --
+# --  apio upload
+# --
+# --  The default flags are used. So the package tq144 is used and the
+# --  bitstream is generated again
+# --
+# -- In order for it to work correctly, we should pass the same flags than
+# -- in the build target:
+# --
+# --  apio upload --pack vq100
 
 
 @cli.command('upload')
-def upload():
+@click.pass_context
+@click.option('--board', type=unicode, metavar='package',
+              help='Set the FPGA board')
+@click.option('--pack', type=unicode, metavar='package',
+              help='Set the FPGA package')
+@click.option('--type', type=unicode, metavar='type',
+              help='Set the FPGA type (hx/lp)')
+@click.option('--size', type=unicode, metavar='size',
+              help='Set the FPGA type (1k/8k)')
+def upload(ctx, board, pack, type, size):
     """Upload bitstream to FPGA."""
-    SCons().run(['upload'])
+
+    # -- Get the variables and change them in the form 'flag=value'
+    vars = format_vars(board, pack, type, size)
+
+    SCons().run(['upload'] + vars)
 
 
 @cli.command('time')
